/*******************************************************************************
Title:      intstr
Desc.:      Converts an integer to a string (only decimal format)
Version:    V0.0.1
Date:       08.04.2013
Author:     bestucki
Licence:    GNU GENERAL PUBLIC LICENSE, Version 3, 29 June 2007

            Copyright 2013 bestucki

            This program is free software: you can redistribute it and/or modify
            it under the terms of the GNU General Public License as published by
            the Free Software Foundation, either version 3 of the License, or
            (at your option) any later version.

            This program is distributed in the hope that it will be useful, but
            WITHOUT ANY WARRANTY; without even the implied warranty of
            MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
            General Public License for more details.

            You should have received a copy of the GNU General Public License
            along with this program. If not, see <http://www.gnu.org/licenses/>.

*******************************************************************************/

#include "intstr.h"

#define NegativeTest(source,possource,negative)                                \
{                                                                              \
  if((source)<0){                                                              \
    (possource)=-(source);                                                     \
    (negative)=1;                                                              \
  }                                                                            \
  else{                                                                        \
    (possource)=(source);                                                      \
    (negative)=0;                                                              \
  }                                                                            \
}

#define PositiveIntegerToString(target,possource,i)                            \
{                                                                              \
  do{                                                                          \
    (target)[(i)++]=(char)((possource)%10)+'0';                                \
    (possource)/=10;                                                           \
  }while(possource);                                                           \
}

#define WriteMinus(target,negative)                                            \
{                                                                              \
  if(negative){                                                                \
    (target)[(i)++]='-';                                                       \
  }                                                                            \
}

#define FinishString(target,i)                                                 \
{                                                                              \
  (target)[(i)]='\0';                                                          \
  strswap(target,i);                                                           \
}

#define AfterCommaToString(target,possource,i,cp)                              \
{                                                                              \
  if((cp)>0){                                                                  \
    while((i)<(cp)){                                                           \
      (target)[(i)++]=(char)((possource)%10)+'0';                              \
      (possource)/=10;                                                         \
    }                                                                          \
    (target)[(i)++]='.';                                                       \
  }                                                                            \
  else{                                                                        \
    (cp)=-(cp);                                                                \
    while((i)<(cp)){                                                           \
      (target)[(i)++]='0';                                                     \
    }                                                                          \
  }                                                                            \
}

#define intXstr(target,source,possource,negative,i)                            \
{                                                                              \
  NegativeTest(source,possource,negative);                                     \
  PositiveIntegerToString(target,possource,i);                                 \
  WriteMinus(target,negative);                                                 \
  FinishString(target,i);                                                      \
}

#define fixintXstr(target,source,cp,possource,negative,i)                      \
{                                                                              \
  NegativeTest(source,possource,negative);                                     \
  AfterCommaToString(target,possource,i,cp);                                  \
  PositiveIntegerToString(target,possource,i);                                 \
  WriteMinus(target,negative);                                                 \
  FinishString(target,i);                                                      \
}

#define uintXstr(target,source,i)                                              \
{                                                                              \
  PositiveIntegerToString(target,source,i);                                    \
  FinishString(target,i);                                                      \
}

#define fixuintXstr(target,source,cp,i)                                        \
{                                                                              \
  AfterCommaToString(target,source,i,cp);                                      \
  PositiveIntegerToString(target,source,i);                                    \
  FinishString(target,i);                                                      \
}

static void strswap(char *str, uint_fast8_t lenght){
  uint_fast8_t i, j, criteria;
  char temp;
  criteria=lenght/2;
  for(i=0,j=lenght-1;i<criteria;i++,j--){
    temp=str[i];
    str[i]=str[j];
    str[j]=temp;
  }
}

char *strRalign(char *str, uint_fast8_t minlenght){
  uint_fast8_t lenght=0;
  while(str[lenght]!='\0'){
    lenght++;
  }
  if(minlenght>lenght){
    strswap(str,lenght);
    while(lenght<minlenght){
      str[lenght++]=' ';
    }
    str[minlenght]='\0';
    strswap(str,minlenght);
  }
  return str;
}

#ifdef INT8_MAX
char *int8str(char *target, int8_t source){
  uint_fast8_t i=0, negative;
  uint_fast8_t possource;
  intXstr(target,source,possource,negative,i);
  return target;
}

char *fixint8str(char *target, int8_t source, int_fast8_t cp){
  uint_fast8_t i=0, negative;
  uint_fast8_t possource;
  fixintXstr(target,source,cp,possource,negative,i);
  return target;
}
#endif /* INT8_MAX */

#ifdef UINT8_MAX
char *uint8str(char *target, uint8_t source){
  uint_fast8_t i=0;
  uintXstr(target,source,i);
  return target;
}

char *fixuint8str(char *target, uint8_t source, int_fast8_t cp){
  uint_fast8_t i=0;
  fixuintXstr(target,source,cp,i);
  return target;
}
#endif /* UINT8_MAX */

#ifdef INT16_MAX
char *int16str(char *target, int16_t source){
  uint_fast8_t i=0, negative;
  uint_fast16_t possource;
  intXstr(target,source,possource,negative,i);
  return target;
}

char *fixint16str(char *target, int16_t source, int_fast8_t cp){
  uint_fast8_t i=0, negative;
  uint_fast16_t possource;
  fixintXstr(target,source,cp,possource,negative,i);
  return target;
}
#endif /* INT16_MAX */

#ifdef UINT16_MAX
char *uint16str(char *target, uint16_t source){
  uint_fast8_t i=0;
  uintXstr(target,source,i);
  return target;
}

char *fixuint16str(char *target, uint16_t source, int_fast8_t cp){
  uint_fast8_t i=0;
  fixuintXstr(target,source,cp,i);
  return target;
}
#endif /* UINT16_MAX */

#ifdef INT24_MAX
char *int24str(char *target, int24_t source){
  uint_fast8_t i=0, negative;
  uint_fast24_t possource;
  intXstr(target,source,possource,negative,i);
  return target;
}

char *fixint24str(char *target, int24_t source, int_fast8_t cp){
  uint_fast8_t i=0, negative;
  uint_fast24_t possource;
  fixintXstr(target,source,cp,possource,negative,i);
  return target;
}
#endif /* INT24_MAX */

#ifdef UINT24_MAX
char *uint24str(char *target, uint24_t source){
  uint_fast8_t i=0;
  uintXstr(target,source,i);
  return target;
}

char *fixuint24str(char *target, uint24_t source, int_fast8_t cp){
  uint_fast8_t i=0;
  fixuintXstr(target,source,cp,i);
  return target;
}
#endif /* UINT24_MAX */

#ifdef INT32_MAX
char *int32str(char *target, int32_t source){
  uint_fast8_t i=0, negative;
  uint_fast32_t possource;
  intXstr(target,source,possource,negative,i);
  return target;
}

char *fixint32str(char *target, int32_t source, int_fast8_t cp){
  uint_fast8_t i=0, negative;
  uint_fast32_t possource;
  fixintXstr(target,source,cp,possource,negative,i);
  return target;
}
#endif /* INT32_MAX */

#ifdef UINT32_MAX
char *uint32str(char *target, uint32_t source){
  uint_fast8_t i=0;
  uintXstr(target,source,i);
  return target;
}

char *fixuint32str(char *target, uint32_t source, int_fast8_t cp){
  uint_fast8_t i=0;
  fixuintXstr(target,source,cp,i);
  return target;
}
#endif /* UINT32_MAX */

#ifdef INT64_MAX
char *int64str(char *target, int64_t source){
  uint_fast8_t i=0, negative;
  uint_fast64_t possource;
  intXstr(target,source,possource,negative,i);
  return target;
}

char *fixint64str(char *target, int64_t source, int_fast8_t cp){
  uint_fast8_t i=0, negative;
  uint_fast64_t possource;
  fixintXstr(target,source,cp,possource,negative,i);
  return target;
}
#endif /* INT64_MAX */

#ifdef UINT64_MAX
char *uint64str(char *target, uint64_t source){
  uint_fast8_t i=0;
  uintXstr(target,source,i);
  return target;
}

char *fixuint64str(char *target, uint64_t source, int_fast8_t cp){
  uint_fast8_t i=0;
  fixuintXstr(target,source,cp,i);
  return target;
}
#endif /* UINT64_MAX */

#undef NegativeTest
#undef PositiveIntegerToString
#undef WriteMinus
#undef FinishString
#undef AfterCommaToString
#undef intXstr
#undef fixintXstr
#undef uintXstr
#undef fixuintXstr

/* end of file */
